import { OrgChart } from "d3-org-chart";

interface Person {
    id: string;
    firstName: string;
    lastName: string;
}

const myData: Person[] = [
    {
        id: "1",
        firstName: "Alice",
        lastName: "Smith",
    },
    {
        id: "2",
        firstName: "Bob",
        lastName: "Smith",
    },
];

const chart = new OrgChart<Person>()
    .container("#my-container")
    .svgHeight(1000)
    .data(myData)
    .nodeHeight(d => (d.data.firstName === "Alice" ? 200 : 100))
    .nodeWidth(d => {
        // $ExpectType HierarchyNode<Person>
        d;
        // $ExpectType Person
        d.data;
        // $ExpectType string
        d.data.firstName;
        // @ts-expect-error
        d.data.middleName;
        // @ts-expect-error
        d.firstName;
        if (d.depth === 0) return 500;
        return 330;
    })
    .nodeId(d => ("firstName" in d ? d.firstName : d.data.firstName))
    .childrenMargin(d => 90)
    .compactMarginBetween(d => 65)
    .compactMarginPair(d => 100)
    .neighbourMargin((a, b) => 50)
    .siblingsMargin(d => 100)
    .buttonContent(({ node, state }) => {
        return `<div>Show ${node.children && node.children[0] ? node.children[0].data.firstName : ""}</div>`;
    })
    .nodeContent((d, i, arr, state) => {
        return `<div>#${i}: ${d.data.firstName} ${d.data.lastName}</div>`;
    })
    .nodeUpdate((d, i, arr) => {
        if (d.data.firstName === "Bob") {
            console.log(`Updated Bob when there are ${i} nodes`);
        }

        if (i === arr.length - 1) {
            console.log(`The last node (${d.data.firstName} ${d.data.lastName}) was updated`);
        }
    })
    .onNodeClick((d) => {
        if (d.id) {
            console.log(`The node with id of ${d.id} selected`);
        }
    })
    .linkUpdate((d, i, arr) => {
        if (d.data.firstName === "Bob") {
            console.log(`Updated Bob when there are ${i} nodes`);
        }

        if (i === arr.length - 1) {
            console.log(`The last node (${d.data.firstName} ${d.data.lastName}) was updated`);
        }
    })
    .render();

// $ExpectType OrgChart<Person>
chart;

// $ExpectType Person[] | null
chart.data();

// $ExpectType Person[] | null
chart.getChartState().data;

// $ExpectType OrgChart<Person>
chart.nodeHeight(node => node.data.firstName.length);

// $ExpectType string
chart.backgroundColor();

chart
    .backgroundColor()
    // @ts-expect-error
    .render();

// $ExpectType OrgChart<Person>
chart.backgroundColor("#eee").render();

// $ExpectType OrgChart<Person>
chart.addNode({ id: "3", firstName: "Charlie", lastName: "Brown" });

// $ExpectType OrgChart<Person>
chart.expandAll();

// $ExpectType OrgChart<Person>
chart.collapseAll();

// $ExpectType number
chart.getChartState().lastTransform.x;

// $ExpectType string[]
chart.getChartState().svg.data();

// $ExpectType Selection<SVGSVGElement, string, null, undefined>
chart.getChartState().svg.on("wheel.zoom", null);

// $ExpectType ZoomBehavior<Element, Person>
chart.zoomBehavior();
