import { FileSaverOptions, saveAs } from "file-saver";
import saveAsReq = require("file-saver");

/**
 * @summary Test for "saveAs" function.
 */
function testSaveAs() {
    const data: Blob = new Blob(["Hello, world!"], { type: "text/plain;charset=utf-8" });
    const filename = "hello world.txt";
    const options: FileSaverOptions = {
        autoBom: false,
    };

    saveAs(data, filename, options);
    saveAsReq(data, filename, options);
}

/**
 * @summary Test for deprecated "saveAs" function.
 */
function testDeprecatedSaveAs() {
    const data: Blob = new Blob(["Hello, world!"], { type: "text/plain;charset=utf-8" });
    const filename = "hello world.txt";
    const disableAutoBOM = true;

    saveAs(data, filename, disableAutoBOM);
    saveAsReq(data, filename, disableAutoBOM);
}

/**
 * @summary Test for "saveAs" function on the window object.
 */
function testWindowSaveAs() {
    const data: Blob = new Blob(["Hello, world!"], { type: "text/plain;charset=utf-8" });
    const filename = "hello world.txt";
    const options: FileSaverOptions = {
        autoBom: false,
    };

    window.saveAs(data, filename, options);
}

/**
 * @summary Test for "saveAs" function with URL as first argument.
 */
function testUrlSaveAs() {
    const url = "https://example.com/test.txt";
    const filename = "hello world.txt";
    const options: FileSaverOptions = {
        autoBom: false,
    };

    window.saveAs(url, filename, options);
}

/**
 * @summary Test for "saveAs" function with the 3rd parameter omitted
 */
function testOptionalOneParamSaveAs() {
    const data: Blob = new Blob(["Hello, world!"], { type: "text/plain;charset=utf-8" });
    const filename = "hello world.txt";
    saveAs(data, filename);
    saveAsReq(data, filename);
}

/**
 * @summary Test for "saveAs" function with the 2nd and 3rd parameters omitted
 */
function testOptionalTwoParamsSaveAs() {
    const data: Blob = new Blob(["Hello, world!"], { type: "text/plain;charset=utf-8" });
    saveAs(data);
    saveAsReq(data);
}
